# 	TkfPW Tcl Utilities

proc tk { option args } {
    switch -- $option {
	colormodel {
	    if {[winfo depth [lindex $args 0]] > 4} {
		return color
	    } else {
		return monochrome
	    }
	}
	default {
	    return [eval {tk-orig $option} $args]
	}
    }
}

proc FindRep {startDir} {

	global Obj_Rep icount

#	show-status "WAIT PLEASE : Scan for Rep" main-status

	set icount 0
	FindRep_recursive $startDir
	set Obj_Rep(number) $icount

#	show-status "Ready..." main-status
}


proc FindRep_recursive {startDir} {

	global Obj_Rep icount

	set pwd [pwd]
	if [catch {cd $startDir} err] {
		PRINT "proc FindRep_recursive : $err"
		return
	}
	set list_files [glob  -nocomplain -- $startDir/*]
	foreach file $list_files {
		if [file isdirectory $file] {
			incr icount
			set Obj_Rep($icount) $file
			FindRep_recursive $file
		}
	}
	cd $pwd
}


proc CheckCommand {command} {
# Check for an executable command
	global env

	if { $command == "" } { return 0 }


	switch -- [llength $command] {

		1 {
			# Single command
			# Absolute name ?
			if [file executable $command] { return 1 }

			# Search in the PATH directories
			set flag 0
			foreach dir [split $env(PATH) :] {
				if [file executable $dir/$command] {
					set flag 1
					set command $dir/$command
					break
				}
			}
			if ![file executable $command] { set flag 0 }
		}


		default {
			# Composite Command
			# such as {nedit 2>/dev/null}
			# or {emacs -geometry 30x30}, etc...
			if [catch {eval exec $command & } CmdPID] {
				set flag 0
			} else {
				exec kill $CmdPID
				set flag 1
			}
	
		}

	}

	return $flag
}
proc newer { file1 file2 } {
	if ![file exists $file1] {
		return 1
	} else {
		if ![file exists $file2] {
			return 0
		} else {
			expr [file mtime $file1] > [file mtime $file2]
		}
	}
}
proc makedir { pathname } {
	if {[file isdirectory $pathname]} {
		return $pathname
	} elseif {[file exists $pathname]} {
		error "Non-directory $pathname already exists."
	} else {
		# Recurse to create intermediate directories
		makedir [file dirname $pathname]
		exec mkdir $pathname
		return $pathname
	}
}
proc fileeq { path1 path2 } {
	file stat $path1 stat1
	file stat $path2 stat2
	if { ($stat1(ino) == $stat2(ino)) && ($stat1(dev) == $stat2(dev))} {
		return 1
	} else {
		return 0
	}
}
proc FindFile { startDir namePat } {
	set pwd [pwd]
	if [catch {cd $startDir} err] {
		PRINT "proc FindFile : $err"
		return
	}
	foreach match [glob -nocomplain -- $namePat] {
		PRINT "proc FindFile : $startDir/$match"
	}
	foreach file [glob -nocomplain *] {
		if [file isdirectory $file] {
			FindFile $startDir/$file $namePat
		}
	}
	cd $pwd
}
proc ldelete { list value } {
	set ix [lsearch -exact $list $value]
	if {$ix >= 0} {
		return [lreplace $list $ix $ix]
	} else {
		return $list
	}
}
proc CheckForPositiveNumber {number} {
	if { $number == "" } {return 0}
	if { $number <= 0 } {return 0}
	if [catch {expr $number + 1} ] { return 0 }
	return 1
}
proc ItemIsInList {item list} {
# Item Is in List ?
	foreach list_item $list {
		if [ string match $list_item $item ] { return 1 }
	}
	return 0
}
proc ItemIsNotInList {item list} {
# Item in Not in List ?
	foreach list_item $list {
		if [ string match $list_item $item ] { return 0 }
	}
	return 1
}
#------------------------------------------------------------------------------#
proc center_window {win} {
	wm withdraw $win
	update idletasks
	set w [winfo reqwidth $win]
	set h [winfo reqheight $win]
	set sh [winfo screenheight $win]
	set sw [winfo screenwidth $win]
	wm geometry $win +[expr {($sw-$w)/2}]+[expr {($sh-$h)/2}]
	wm deiconify $win
}
proc EnableButtonInW { w } {
#Enable all buttons children in window w
	foreach button [winfo children $w] {
		$button configure -state normal
	}
}
proc DisableButtonInW { w } {
#Disable all buttons children in window w
	foreach button [winfo children $w] {
		$button configure -state disabled
	}
}
#------------------------------------------------------------------------------#

proc InProgress_create {self steps} {
	global progress progress_slot

	set progress_slot($self,steps) $steps
	set progress_slot($self,title) {}
	set progress_slot($self,barwidth) 200
	set progress_slot($self,barheight) 20
	set progress_slot($self,barcolor) DodgerBlue
	set progress_slot($self,step) 0

	toplevel $self
	wm title $self {TkfPW in progress ...}

	set wf $self.f
	frame $wf

	$self config -bd 4 -relief ridge
	label $wf.title -bd 2 -relief ridge
	pack $wf.title -side top -padx 2 -pady 2
	frame $wf.spacer -width $progress_slot($self,barwidth)
	frame $wf.bar -relief raised -bd 2 -highlightthickness 0 -height $progress_slot($self,barheight) -bg $progress_slot($self,barcolor)
	pack  $wf.spacer $wf.bar -side top -padx 10 -anchor w
	label $wf.percentage -text 0%
	pack $wf.percentage -side top -fill x -padx 2 -pady 2

	pack $wf -expand true -fill both

    	wm transient $self [winfo toplevel [winfo parent $self]]
	center_window $self

	set progress($self,title) $wf.title
	set progress($self,spacer) $wf.spacer
	set progress($self,bar) $wf.bar
	set progress($self,percentage) $wf.percentage

        catch {grab set $self}
}

proc InProgress_update { self title step } {
	global progress progress_slot

	set progress_slot($self,step) $step
	set progress_slot($self,title) $title

        set fraction [expr 1.0*$progress_slot($self,step)/$progress_slot($self,steps)]

        $progress($self,title) \
		config -text $progress_slot($self,title)
        $progress($self,percentage) \
		config -text [format %.0f%% [expr 100.0*$fraction]]
        $progress($self,bar) \
		config -width [expr int($progress_slot($self,barwidth)*$fraction)]

	if { $progress_slot($self,step) >= $progress_slot($self,steps) } {	
        	$progress($self,title) config -text "Done !"
		update idletasks
		after 500
        	if {[grab current $self] == $self} {
			grab release $self
		}
		catch {destroy $self}
		set progress(done) 1
		return
	}

	# Force a display update
	update idletasks
}
proc InProgress { parent steps title step} {
	global progress progress_slot

	if ![winfo exists $parent] { set parent "" }
	set w $parent.in-progress

	if ![winfo exists $w] {
		InProgress_create $w $steps
	}
	InProgress_update $w $title $step
}
#------------------------------------------------------------------------------#
proc ScrolledListbox { winlist args } {
# Create listbox attached to scrollbars, pass thru $args
	# - All widgets are imbeded into the main frame "winlist"
	# - "args" are the customised arguments attached to
	#   the returned listbox name "winlist.list".

	# nb: I prevent scrollbars from accepting the focus
	#	during keyboard traversal (e.g., Tab and Shift-Tab)

 	catch {frame $winlist}

	eval {listbox $winlist.list \
		-yscrollcommand [list $winlist.sy set] \
		-xscrollcommand [list $winlist.sx set]} $args
	scrollbar $winlist.sy -orient vertical -cursor {sb_v_double_arrow} \
		-command [list $winlist.list yview] -takefocus 0
	# Create extra frame to hold pad and horizontal scrollbar
	frame $winlist.bottom
	scrollbar $winlist.sx -orient horizontal -cursor {sb_h_double_arrow} \
		-command [list $winlist.list xview] -takefocus 0
	# Create padding based on the scrollbar width and border
	set pad [expr [$winlist.sy cget -width] + 2* \
		([$winlist.sy cget -bd] + \
		 [$winlist.sy cget -highlightthickness])]
	frame $winlist.pad -width $pad -height $pad
	# Arrange everything in the winlist frame
	pack $winlist.bottom -side bottom -fill x
	pack $winlist.pad -in $winlist.bottom -side right
	pack $winlist.sx -in $winlist.bottom -side bottom -fill x
	pack $winlist.sy -side right -fill y
	pack $winlist.list -side left -fill both -expand true
	pack $winlist -side top -fill both -expand true
	return $winlist.list
}
proc ScrolledText { wintext args } {
# Create textbox attached to scrollbars, pass thru $args
	# - All widgets are imbeded into the main frame "wintext"
	# - "args" are the customised arguments attached to
	#   the returned listbox name "wintext.text".

	# nb: I prevent scrollbars from accepting the focus
	#	during keyboard traversal (e.g., Tab and Shift-Tab)

 	catch {frame $wintext}

	eval {text $wintext.text \
		-yscrollcommand [list $wintext.sy set] \
		-xscrollcommand [list $wintext.sx set]} $args
	scrollbar $wintext.sy -orient vertical -cursor {sb_v_double_arrow} \
		-command [list $wintext.text yview]
	# Create extra frame to hold pad and horizontal scrollbar
	frame $wintext.bottom
	scrollbar $wintext.sx -orient horizontal -cursor {sb_h_double_arrow} \
		-command [list $wintext.text xview]
	# Create padding based on the scrollbar width and border
	set pad [expr [$wintext.sy cget -width] + 2* \
		([$wintext.sy cget -bd] + \
		 [$wintext.sy cget -highlightthickness])]
	frame $wintext.pad -width $pad -height $pad
	# Arrange everything in the wintext frame
	pack $wintext -side top -fill both -expand true
	pack $wintext.bottom -side bottom -fill x
	pack $wintext.pad -in $wintext.bottom -side right
	pack $wintext.sx -in $wintext.bottom -side bottom -fill x
	pack $wintext.sy -side right -fill y
	pack $wintext.text -side left -fill both -expand true
	return $wintext.text
}
proc ScrolledCanvas { c args } {
# Create Canvas attached to scrollbars, pass thru $args
	# - All widgets are imbeded into the main frame "c"
	# - "args" are the customised arguments attached to
	#   the returned canvas name "c.canvas".

 	catch {frame $c}

	eval {canvas $c.canvas \
		-xscrollcommand [list $c.sx set] \
		-yscrollcommand [list $c.sy set] } $args
	scrollbar $c.sy -orient vertical \
		-command [list $c.canvas yview]
	# Create extra frame to hold pad and horizontal scrollbar
	frame $c.bottom
	scrollbar $c.sx -orient horizontal \
		-command [list $c.canvas xview]
	# Create padding based on the scrollbar width and border
	set pad [expr [$c.sy cget -width] + 2* \
		([$c.sy cget -bd] + \
		 [$c.sy cget -highlightthickness])]
	frame $c.pad -width $pad -height $pad
	# Arrange everything in the canvas frame
	pack $c.bottom -side bottom -fill x
	pack $c.pad -in $c.bottom -side right
	pack $c.sx -in $c.bottom -side bottom -fill x
	pack $c.sy -side right -fill y
	pack $c.canvas -side left -fill both -expand true
	pack $c -side top -fill both -expand true
	return $c.canvas
}
#------------------------------------------------------------------------------#
proc WriteVarInFile {var file} {
#Write var in file
	# Open a stream for file
	if [catch {open $file w} fileId] {
		PRINT "proc WriteVarInFile : $fileId"
	} else {
		# Put var in file
		puts $fileId $var
		close $fileId
	}
}
proc ReadInFile {file} {
#Return the file contents
	# Open a stream for file
	if [catch {open $file r} fileId] {
		PRINT "proc ReadInFile :$fileId"
		return {}
	} else {
		# Get var from file
		catch { read $fileId } var
		close $fileId
		return $var
	}
}
proc SetValFlag01 {valflag} {
#In valflag, change the flag to 0 (off) or 1 (on), default is off
#the type of valflag is "val@flag", if separator is @

	set val	 [GetVal_ValFlag  $valflag]
	set flag [GetFlag_ValFlag $valflag]
	switch -glob -- $flag {
		{[Oo][Nn]*}	{set flag01 1}
		{[Yy]*}		{set flag01 1}
		{[Oo][Uu][Ii]*}	{set flag01 1}
		{[Tt]*}		{set flag01 1}
		{+*} 		{set flag01 1}
		{1*} 		{set flag01 1}
		default 	{set flag01 0}
	}
	return "$val@$flag01"
}
proc SetListValFlag01 {listvalflag} {
	set listvalflag01 {}
	foreach valflag $listvalflag {
		lappend listvalflag01 [SetValFlag01 $valflag]
	}
	return $listvalflag01
}
proc GetListVal_FlagOn {listvalflag} {
# return a list of the flag On (i.e. =1) values

	set listval_flagon {}
	foreach valflag $listvalflag {
	 if [GetFlag_ValFlag $valflag] {lappend listval_flagon [GetVal_ValFlag $valflag] }
	}
	return $listval_flagon
} 
proc CleanListValFlag {listvalflag} {
	set listvalflag_clean {}
	foreach valflag $listvalflag {
		lappend listvalflag_clean [GetVal_ValFlag $valflag]@[GetFlag_ValFlag $valflag]
	}
	return $listvalflag_clean
}
proc GetVal_ValFlag {valflag} {
	return [ string trim [lindex  [split $valflag @] 0] ]
}

proc GetFlag_ValFlag {valflag} {
	return [ string trim [lindex  [split $valflag @] 1] ]
}
proc AppendItemInList { list item  } {
# Append the item in list if item is not in list
# and return the new list
	if { [lsearch -exact $list $item] < 0 } {
		lappend list $item
	}
	return $list
}
proc  RemoveDuplicateItemsInList { list } {
# return a list in which duplicated items where removed
	set list_uniq {}
	foreach item $list {
		if { [lsearch -exact $list_uniq $item ] <0} {
			lappend list_uniq $item
		}
	}
	return $list_uniq
}
proc ExtractInList { list listarg } {
# Find all valid elem of List "listarg" in List "list"
# and return the result in list "listfind"
# Name Completion is done on "listarg" and arguments remain unchanged.
	set listfind ""
	foreach item $listarg {
	set ltmp $list; set go_on true
		while {$go_on} {
			set index [lsearch $ltmp $item]
			if { $index < 0 } {
				set go_on false
			} else {
				set itemfind  [lindex $ltmp $index ]
				set ltmp [lrange $ltmp [ expr $index +1] end ]
				if { [lsearch $listfind $itemfind] <0 } {
					lappend listfind $itemfind
				}
			}
		}
	}
	return $listfind
}

proc AppendInList { list listarg } {
# Append all valid elem of List "listarg" with List "list"
# and return le result in list "newlist".
# Name Completion is done on "listarg" and arguments remain unchanged.
	set newlist $list
	foreach item $listarg {
		if { [lsearch $newlist $item] <0 } {
			lappend newlist $item
		}
	}
	return $newlist
}
#------------------------------------------------------------------------------#
proc ShowStatus {com w} {
	show-status $com $w
}
proc show-status {message status} {
	global Obj_Mes
	set Obj_Mes($status) $message
	update idletasks
}
#------------------------------------------------------------------------------#
proc SelectionInListBox { w } {
# return the selected element in listbox w
	set index [$w curselection]
	if { $index >=0 } {
		return [$w get $index]
	} else {
		return {}
	}
}
#------------------------------------------------------------------------------#
proc EditFile {parent file} {
# Edit a file and return : 
#	- the Edit Process Pid if Ok
#	- 0 if not.

	global Glob_Pref
	if ![winfo exists $parent] { set parent "" }

	if [catch {eval exec $Glob_Pref(fPWEditor) $file &} EditPID] {
		tk_dialog $parent.d {TkfPW Error in Editor} "$EditPID" error 0 {???}
		return 0
	} else {
		PRINT "Editing file $file : PID $EditPID"
		return $EditPID
	}
}
#-----------------------------------------------------------------------
proc EditFileNew {parent file bfile} {
global editor Stream_EditFile Pid_EditFile wprint
global Glob_Pref env

	if [info exists Stream_EditFile($file)] {
		PRINT "proc EditFile : file $file already opened !"
		return
	}
	set edit $env(TKFPW_HOME)/cmd/edit.tcl
	if [catch {open "| tclsh $edit [list $Glob_Pref(fPWEditor) $file] |& cat"} id] {
		PRINT "Error in proc EditFile : $id"
		return
	} else {
		fileevent $id readable [list EditFile_Reader $file $bfile $id]
		set Stream_EditFile($file) $id
		PRINT "proc EditFile : open stream $id for file $file"
		catch {$bfile configure -state disabled}
	}
}

proc EditFile_Reader {file bfile id} {
global Stream_EditFile Pid_EditFile wprint
	if [eof $id] {
		PRINT "EditFile_Reader : EOF"
		PRINT "EditFile_Reader : End of Editing for file $file"
		catch {close $id}
		catch {unset Stream_EditFile($file)}
		catch {unset Pid_EditFile($file)}
		catch {$bfile configure -state normal}
		return
	}
	gets $id line
#	PRINT "EditFile_Reader : $line"
	if [string match *PID=* $line] {
		set Pid_EditFile($file) [lindex [split $line =] 1]
		PRINT "EditFile_Reader : Got the PID:$Pid_EditFile($file)"
	}
}

proc EditFile_Close {list_files} {
global Stream_EditFile Pid_EditFile wprint
	set icanquit 1
	set list_openedfiles {}
	set list_pid {}
	foreach file $list_files {
		if [info exists Stream_EditFile($file)] {
			set icanquit 0
			lappend list_openedfiles [file tail $file]
			catch {lappend list_pid $Pid_EditFile($file)} err
		}
	}

	if {$icanquit} {
		exit
	} else {
		PRINT "proc EditFile_Quit : files still opened !"
		PRINT $list_openedfiles
		PRINT $list_pid
		if [tk_dialog .info {TkfPW Confirm} "Close Opened Files ?" questhead 1 Yes No] {return}
		if [ catch {exec kill $list_pid  } err ] {
			PRINT "proc EditFile_Quit : process(es) not killed !!!"
			return
		} else {
			PRINT "proc EditFile_Quit : ok remaining process(es) killed"
			catch {unset Stream_EditFile Pid_EditFile}

			PRINT "Bye ..."
			update idletasks
			after 2000
			exit
		}
	}
	
}
#------------------------------------------------------------------------------#
proc HideSlavesInWidget {w} {
# Recursive Disable state of slaves in w
	foreach slave [pack slaves $w] {
		catch {$slave configure -state disabled}
		HideSlavesInWidget $slave
	}
}
proc ShowSlavesInWidget {w} {
# Recursive Normal state of slaves in w
	foreach slave [pack slaves $w] {
		catch {$slave configure -state normal}
		ShowSlavesInWidget $slave
	}
}
#------------------------------------------------------------------------------#
proc busy {cmds} {
# Faire apparaitre un curseur "watch"
# pendant l'execution de la commande "cmds"
# Tire de Tk 4+ Usage FAQ : 
# http://www.cs.uoregon.edu/research/tcl/faqs
	global errorInfo

	set busy {.app .root}
	set list [winfo children .]
	while {$list != ""} {
		set next {}
		foreach w $list {
			set class [winfo class $w]
			set cursor [lindex [$w config -cursor] 4]
			if {[winfo toplevel $w] == $w || $cursor != ""} {
				lappend busy [list $w $cursor]
			}
			set next [concat $next [winfo children $w]]
		}
		set list $next
	}

	foreach w $busy {
		catch {[lindex $w 0] config -cursor watch}
	}

	update idletasks

	set error [catch {uplevel eval [list $cmds]} result]
	set ei $errorInfo

	foreach w $busy {
		catch {[lindex $w 0] config -cursor [lindex $w 1]}
	}

	if $error {
		error $result $ei
	} else {
		return $result
	}
}
#------------------------------------------------------------------------------#
